Package: lhs
Title: Latin Hypercube Samples
Version: 1.3.0
Authors@R: 
    person(given = "Rob",
           family = "Carnell",
           role = c("aut", "cre"),
           email = "bertcarnell@gmail.com",
           comment = c(ORCID = "0009-0009-0465-7564")
		   )
Description: Provides a number of methods for creating and augmenting Latin Hypercube Samples and Orthogonal Array Latin Hypercube Samples.
License: GPL-3
Encoding: UTF-8
Depends: R (>= 4.1.0)
LinkingTo: Rcpp
Imports: Rcpp
Suggests: testthat, DoE.base, knitr, rmarkdown
URL: https://github.com/bertcarnell/lhs
BugReports: https://github.com/bertcarnell/lhs/issues
RoxygenNote: 7.3.3
VignetteBuilder: knitr
NeedsCompilation: yes
Packaged: 2026-04-27 01:36:37 UTC; bertc
Author: Rob Carnell [aut, cre] (ORCID: <https://orcid.org/0009-0009-0465-7564>)
Maintainer: Rob Carnell <bertcarnell@gmail.com>
Repository: CRAN
Date/Publication: 2026-04-27 05:10:09 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-04-27 05:18:24 UTC; unix
Archs: lhs.so.dSYM
