# Generated by roxygen2: do not edit by hand

export(augmentLHS)
export(correlatedLHS)
export(createAddelKemp)
export(createAddelKemp3)
export(createAddelKempN)
export(createBose)
export(createBoseBush)
export(createBoseBushl)
export(createBush)
export(createBusht)
export(create_galois_field)
export(create_oalhs)
export(geneticLHS)
export(get_library_versions)
export(improvedLHS)
export(maximinLHS)
export(oa_to_oalhs)
export(optAugmentLHS)
export(optSeededLHS)
export(optimumLHS)
export(poly2int)
export(poly_prod)
export(poly_sum)
export(qdirichlet)
export(qfactor)
export(qinteger)
export(randomLHS)
export(runifint)
import(Rcpp)
importFrom(stats,dist)
importFrom(stats,na.exclude)
importFrom(stats,na.omit)
importFrom(stats,qgamma)
importFrom(stats,runif)
importFrom(utils,packageVersion)
useDynLib(lhs)
